package com.climber;

import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector2;

/**
 * Collision Class
 * @author Zak
 *
 * This class was first implemented to be used while detecting collisions between the character and the blocks on the screen.
 * 
 * It works by calculating the distances between the middles of the squares. I separate them between X and Y distances so that they can be compared
 * against each other to see whether or not the blocks are colliding on the X axis(left and right) or the Y axis(up and down).
 * 
 * First though, it checks to see where the two blocks are in relation to each other. For example, if the middle of the character is above the middle
 * of the block, then there is no way that the top of the character could be hitting the bottom of the block, so for isCollidingBottom() it returns false right away.
 * 
 * NOTE: This class assumes that the two rectangles are in fact colliding and have already been tested for that. This class is only to be used to help determine
 * which sides of the blocks are colliding, not whether or not they are colliding.
 */
public class Collision {
	private boolean m_aIsHigher; //true if the middle of a is higher than the middle of b
	private boolean m_aIsToTheRight; //true if the middle of a is to the right of the middle of b
	
	private float xDist;
	private float yDist;
	
	public Collision(Rectangle a, Rectangle b) {
		
		Vector2 m_AMiddle = new Vector2(a.x + (a.width/2.0f), a.y + (a.height/2.0f));
		Vector2 m_BMiddle = new Vector2(b.x + (b.width/2.0f), b.y + (b.height/2.0f));
		
		if ( (a.x + (a.width/2.0f) ) > (b.x + (b.width/2.0f))) {
			m_aIsToTheRight = true;
		} else {
			m_aIsToTheRight = false;
		}
		
		if ( (a.y + (a.height/2.0f)) > (b.y + (b.height/2.0f))) {
			m_aIsHigher = true;
		} else {
			m_aIsHigher = false;
		}
		
		Vector2 x1 = new Vector2(m_AMiddle.x, 0);
		Vector2 x2 = new Vector2(m_BMiddle.x, 0);
		
		Vector2 y1 = new Vector2(0, m_AMiddle.y);
		Vector2 y2 = new Vector2(0, m_BMiddle.y);
		
		xDist = x1.dst(x2);
		yDist = y1.dst(y2);
	}

	public boolean isCollidingBottom() {
		if ( !m_aIsHigher ) {
			//if the middle of A isn't higher than the middle of B, then obviously the bottom isn't colliding
			return false;
		}
		
		if ( xDist < yDist )
			return true;
		return false;
	}

	public boolean isCollidingTop() {
		if ( m_aIsHigher ) {
			//if the middle of A is higher than the middle of B, then obviously the top isn't colliding
			return false;
		}

		if ( xDist < yDist )
			return true;
		return false;
	}

	public boolean isCollidingLeft() {
		if (!m_aIsToTheRight) {
			//if A is to the left of B, then obviously the left side isn't colliding
			return false;
		}
		
		if ( xDist > yDist )
			return true;
		return false;
	}

	public boolean isCollidingRight() {
		if (m_aIsToTheRight) {
			//if A is to the left of B, then obviously the left side isn't colliding
			return false;
		}
		
		if ( xDist > yDist )
			return true;
		return false;
	}
}
